@@ -1,13 +1,14 @@
-use rugraph::rugraph::Graph;
+use rugraph::digraph::digraph_from_dot_string;
+use rugraph::digraph::DiGraph;
+use rugraph::rugraph::IDiGraph;
 use rugraph::rugraph::IGraph;
-use rugraph::rugraph::graph_from_dot_string;
 use std::fs::File;
 
 fn main() {
     println!("Example of dot file creation. Check test1.dot file.\nTo create a picture install graphivz.\n\n$ dot -Tpng example1.dot -o example1.png\n\n");
 
     let mut fd = File::create("example1.dot").expect("error creating file");
-    let mut graph = Graph::<String>::new();
+    let mut graph = DiGraph::<String>::new();
     graph.add_node("a".to_string());
     graph.add_node("b".to_string());
     graph.add_node("c".to_string());
@@ -24,10 +25,12 @@ fn main() {
 
     println!("Creating a new graph from the file content.");
 
-    let graph2 = match graph_from_dot_string(&s)
-    {
-        Err(e) => { println!("Error {e}"); return },
-        Ok(v) => v
+    let graph2 = match digraph_from_dot_string(&s) {
+        Err(e) => {
+            println!("Error {e}");
+            return;
+        }
+        Ok(v) => v,
     };
     println!("Number of nodes of new graph {}", graph2.count_nodes());
 }
